@@ -1,4 +1,6 @@
 import json
+import traceback
+import os
 from datetime import datetime, timezone
 
 
@@ -9,18 +11,33 @@ class Event:
         name: The name of the event
         data: Optional dictionary containing event-specific data
         timestamp: ISO format timestamp, defaults to current UTC time if not provided
-
+        turn: The turn number of the event, defaults to 0
+        background: Whether the event is a background task, defaults to False
+        background_task_name: The name of the background task, defaults to None
     Attributes:
         name: The event name
         data: Event data dictionary
         timestamp: ISO format timestamp string
+        turn: The turn number of the event
+        background: Whether the event is a background task, defaults to False
+        background_task_name: The name of the background task, defaults to None
     """
 
-    def __init__(self, name: str, data: dict = {}, timestamp: str | None = None):
+    def __init__(
+        self,
+        name: str,
+        data: dict = {},
+        timestamp: str | None = None,
+        turn: int = 0,
+        background: bool = False,
+        background_task_name: str = None,
+    ):
         self.name = name
         self.data = data
-        # Default to current UTC time if not provided
         self.timestamp = timestamp or datetime.now(timezone.utc).isoformat()
+        self.turn = turn
+        self.background = background
+        self.background_task_name = background_task_name
 
 
 class CustomEvent(Event):
@@ -97,10 +114,11 @@ class TextEvent(Event):
     Args:
         source: The source of the text event (e.g. "speech", "text")
         text: The text content spoken or typed by the user
+        language: The language of the text (optionally provided by model)
     """
 
-    def __init__(self, source: str, text: str):
-        super().__init__("text", {"source": source, "text": text})
+    def __init__(self, source: str, text: str, language: str | None = None):
+        super().__init__("text", {"source": source, "text": text, "language": language})
 
 
 class TextToSpeechEvent(Event):
@@ -127,6 +145,7 @@ class TextToSpeechEvent(Event):
         speed: Optional playback speed modifier ranging from 0.25x to 4.0x.
             Supported by OpenAI (0.25x-4.0x) and Azure (0.5x-3.0x) voices.
             Defaults to 1.0
+        language: Optional language code for the speech, defaults to "en"
         stream: Optional streaming configuration (tri-state support)
     """
 
@@ -138,6 +157,7 @@ class TextToSpeechEvent(Event):
         interruptible: bool = True,
         instructions="",
         speed=1.0,
+        language="en",
         stream=None,
     ):
         data = {
@@ -147,6 +167,7 @@ class TextToSpeechEvent(Event):
             "interruptible": interruptible,
             "instructions": instructions,
             "speed": speed,
+            "language": language,
         }
 
         # Only include stream if explicitly provided (tri-state support)
@@ -218,16 +239,72 @@ class ErrorEvent(Event):
 class LogEvent(Event):
     """Logs a message to the system logs.
 
-    This event creates a log entry with the specified message,
+    This event creates a log entry with the specified message and file/line number where it is triggered from,
     useful for debugging, monitoring, and tracking system behavior.
 
     Args:
         message: The message to log
     """
 
-    def __init__(self, message: str):
+    def __init__(self, message: str, location: str = None):
         super().__init__("log", {"message": message})
 
+        # If location is provided, use it. Otherwise, try to calculate it
+        if location:
+            self.data["location"] = location
+        else:
+            calculated_location = self._capture_user_location()
+            if calculated_location:
+                self.data["location"] = calculated_location
+
+    def _capture_user_location(self):
+        """Capture the location where this LogEvent was created in user code."""
+        # Get the call stack
+        stack = traceback.extract_stack()
+
+        # Look for the first frame that's user code (not infrastructure)
+        for frame in reversed(stack):
+            filename = frame.filename
+            basename = os.path.basename(filename)
+
+            # Skip infrastructure files
+            if any(
+                skip in filename
+                for skip in [
+                    "runtime.py",  # Our runtime
+                    "site-packages",  # Python packages
+                    "__pycache__",  # Cached Python files
+                    "asyncio",  # Python async machinery
+                    "generator",  # Generator infrastructure
+                    "events.py",  # Event processing infrastructure
+                ]
+            ):
+                continue
+
+            # Skip absolute system paths
+            if filename.startswith("/usr/") or filename.startswith("/System/"):
+                continue
+
+            # Found user code!
+            if basename.endswith(".py"):
+                return f"{basename}:{frame.lineno}"
+
+        # Enhanced fallback: search deeper in stack
+        for i in range(3, min(len(stack), 10)):
+            frame = stack[-i]
+            filename = frame.filename
+            basename = os.path.basename(filename)
+
+            if (
+                basename.endswith(".py")
+                and "site-packages" not in filename
+                and not filename.startswith("/usr/")
+                and not filename.startswith("/System/")
+            ):
+                return f"{basename}:{frame.lineno}"
+
+        return None
+
 
 class CollectPaymentEvent(Event):
     """Event for collecting payment from the user.
@@ -445,7 +522,7 @@ class TurnInterruptedEvent(Event):
     """
 
     def __init__(self):
-        super().__init__("turn_interrupted", {})
+        super().__init__("turn_interrupted")
 
 
 class InitializeEvent(Event):
@@ -477,6 +554,16 @@ class InitializeEvent(Event):
         )
 
 
+class HandlerEndEvent(Event):
+    """Event for when a handler is finished.
+
+    This event is for internal use only. It is used to indicate that a handler has finished.
+    """
+
+    def __init__(self):
+        super().__init__("handler_end")
+
+
 class TestingEvent(Event):
     """Event for testing purposes.
 
@@ -629,6 +716,26 @@ class TransferSessionEvent(Event):
         super().__init__("transfer_session", payload)
 
 
+class TranscriptPartEvent(Event):
+    """Event for a transcript part.
+
+    This event is for internal use only. It is used to store transcript parts.
+
+    Args:
+        role: The role of the transcript part
+        content: The content of the transcript part
+    """
+
+    def __init__(self, role: str, content: str):
+        super().__init__(
+            "transcript_part",
+            {
+                "role": role,
+                "content": content,
+            },
+        )
+
+
 def event_to_str(event: Event) -> str:
     """Convert an Event object to its JSON string representation.
 
@@ -638,8 +745,16 @@ def event_to_str(event: Event) -> str:
     Returns:
         JSON string representation of the event
     """
-    payload = {"name": event.name, "data": event.data, "timestamp": event.timestamp}
-    return json.dumps(payload)
+    return json.dumps(
+        {
+            "name": event.name,
+            "data": event.data,
+            "timestamp": event.timestamp,
+            "turn": event.turn,
+            "background": event.background,
+            "background_task_name": event.background_task_name,
+        }
+    )
 
 
 def event_from_str(event_str: str) -> Event:
@@ -658,11 +773,13 @@ def event_from_str(event_str: str) -> Event:
     Raises:
         ValueError: If the event type is unknown or unsupported
     """
-    event = json.loads(event_str)
-    name = event["name"]
-    # Default timestamp to current UTC if missing
-    timestamp = event.get("timestamp") or datetime.now(timezone.utc).isoformat()
-    data = event["data"]
+    event: dict = json.loads(event_str)
+    name = event.get("name")
+    data = event.get("data", {})
+    timestamp = event.get("timestamp", datetime.now(timezone.utc).isoformat())
+    turn = event.get("turn", 0)
+    background = event.get("background", False)
+    background_task_name = event.get("background_task_name", None)
 
     # Dynamic handling for metrics namespace
     if isinstance(name, str) and name.startswith("metrics."):
@@ -686,7 +803,7 @@ def event_from_str(event_str: str) -> Event:
         "context": lambda: ContextUpdateEvent(data.get("context")),
         "error": lambda: ErrorEvent(data.get("message")),
         "interrupt": InterruptEvent,
-        "log": lambda: LogEvent(data.get("message")),
+        "log": lambda: LogEvent(data.get("message"), data.get("location")),
         "silence": lambda: SilenceEvent(data.get("duration")),
         "start": StartEvent,
         "stop": StopEvent,
@@ -697,6 +814,7 @@ def event_from_str(event_str: str) -> Event:
             data.get("interruptible", True),
             data.get("instructions", ""),
             data.get("speed", 1.0),
+            data.get("language", "en"),
             data.get("stream"),
         ),
         "text": lambda: TextEvent(data.get("source"), data.get("text")),
@@ -726,6 +844,7 @@ def event_from_str(event_str: str) -> Event:
             data.get("is_debug"),
             data.get("context"),
         ),
+        "handler_end": HandlerEndEvent,
         "debug": lambda: DebugEvent(
             data.get("event_name"),
             data.get("event_data"),
@@ -748,11 +867,17 @@ def event_from_str(event_str: str) -> Event:
             environment=data.get("environment"),
             data=data.get("data", {}),
         ),
+        "transcript_part": lambda: TranscriptPartEvent(
+            data.get("role"), data.get("content")
+        ),
     }
 
     if name in event_types:
         e = event_types[name]()
         e.timestamp = timestamp
+        e.turn = turn
+        e.background = background
+        e.background_task_name = background_task_name
         return e
 
     raise ValueError(f"Unknown event type: {name}")
